<footer>
    <div class="container-fluid">
        <div class="row center-block footer-text">
            <p class="text-muted text-center">{{ footer.render() }}</p>
        </div>
    </div>
</footer>