<div id="sample-container" class="col-sm-12">
    {{ sample_html }}
    {% if caption %}
        <div class="caption text-center text-muted">
            {{ caption }}
        </div>
    {% endif %}
</div>